#!/usr/bin/python3.6
from . import connector
import sys
import os
import json
import argparse
from subprocess import call
# import logging
# logger = logging.getLogger(__name__)
# logger.setLevel(logging.INFO)
# handler = logging.FileHandler('/tmp/plugin.log')
# logger.info(msg="test")
# breakpoint()
#import ggplot as gg
passDbConnection = True


if __name__ == "__main__":
    # server infos are given as arguments
    server = sys.argv[1]
    database = sys.argv[2]
    username = sys.argv[3]
    password = sys.argv[4]
    driverLocation = sys.argv[5] # /opt/microsoft/msodbcsql17/lib64/libmsodbcsql-17.5.so.1.1
    query = sys.argv[6] 
    thisPluginName = sys.argv[7] 
    cache = {
        "server":server,
        "database":database,
        #"username":username,
        #"password":password,
        "driverLocation":driverLocation,
        "query":query,
        "thisPluginName":thisPluginName,
    }
    if not passDbConnection:
        cursor = connector.getCursorFromDatabase(server, database, username, password, driverLocation)
        print("connnected")
        # existing query that returns all rows from the table for caching
        # 'EXEC [Ultramerchant].dbo.pr_membership_credit_billing_stats_24x7_monitor_sel'
        rows = connector._fetchall(cursor, query)
        for i in range(len(rows)):
            key = rows[i][1]
            value = rows[i][2]
            cache[key] = value
    # # validate data before saving to cache
    # ww = connector.validatePluginData(cache)
    # breakpoint()
    
    # save cache to file as thisPluginName 
    cacheFileName = "/tmp/" + thisPluginName + ".cache" # 
    open(cacheFileName, 'w').write(
        json.dumps(cache, indent=4)
    )
    print(json.dumps(cache, indent=4, sort_keys=True))

    site24x7_location = "/opt/site24x7/monagent/plugins//plugin/plugin.py"
    folderDestination = os.path.join(site24x7_location, thisPluginName)
    fileDestination = os.path.join(folderDestination, thisPluginName + ".py")
    print("folderDestination: " + folderDestination)
    print("fileDestination: " + fileDestination)
    # BELOW PROCESS WILL BE DONE BY JENKINS
    # create a folder if not exists
    if not os.path.exists(folderDestination):
        call(f"mkdir {folderDestination}", shell=True)
        os.mkdir(folderDestination)

    # create a python file if not exists
    if not os.path.isfile(fileDestination):
        call(f"touch {fileDestination}", shell=True)
    open(fileDestination, 'w').write(f'''#!/usr/bin/python3.6
import json
from site24x7_plugin_helper.connector import guessMetricByType
cacheData = json.loads(open("{cacheFileName}").read())
PLUGIN_VERSION = "2"
HEARTBEAT = "true"
FREQUENCY = "1"
METRIC_UNITS = {{}}

def metricCollector():
    data = {{}}
    guessMetricByType(data, cacheData)
    data["plugin_version"] = PLUGIN_VERSION
    data['heartbeat_required'] = HEARTBEAT
    data["CPU"] = 100
    for i in cacheData:
        data[i] = cacheData[i]
    return data

if __name__ == "__main__":
    result = metricCollector()
    print(json.dumps(result, indent=4, sort_keys=False))
''' )



import pyodbc
from abc import ABC, abstractmethod
class DatabaseClient(ABC):
    def __init__(self,server,database,username,password,driverLocation):
        self.server=server
        self.database=database
        self.username=username
        self.password=password
        self.driverLocation=driverLocation
        return
        if self.databaseType().lower() == "mssql":
            self.cnxn = pyodbc.connect(
                "DRIVER="+str(driverLocation)+";"+
                "SERVER="+str(server)+";"+
                "DATABASE="+str(database)+";"+
                "UID="+str(username)+";"+
                "PWD="+str(password)+";"
            , timeout=5)
            self.cursor = self.cnxn.cursor()
        else:
            self.cursor=self.getCursorFromDatabase(
                self.server,
                self.database,
                self.username,
                self.password,
                self.driverLocation
            )

    def getCursor(self):
        return self.cursor

    @abstractmethod
    def databaseType(self):
        return "MSSQL"

    def getCursorFromDatabase(self,
        server:str,
        database:str,
        username:str,
        password:str,
        driverLocation:str
    ):
        raise NotImplementedError("getCursorFromDatabase is not implemented")

    def fetchall(self, query:str):
        self.cursor.execute(query)
        rowAll = self.cursor.fetchall()
        return rowAll

    def fetchone(self, query:str):
        self.cursor.execute(query)
        row = self.cursor.fetchone()
        return row


# import pandas as pd

# def getDataframe(query:str):
#     try:
#         df = pd.read_sql(query, con)
#         return df
#     except Exception as e:
#         print(e)
#         return None
        
def guessMetricByType(mainDict:dict, checkDict:dict):
    if "units" not in mainDict:
        mainDict["units"] = {}
    for i in checkDict:
        try:
            int(checkDict[i])
            mainDict["units"][i] = 'count'
        except ValueError:
            mainDict["units"][i] = 'string'

##validation method - This is only for Output validation purpose
def validatePluginData(result):
     obj,mandatory ={'Errors':""},['heartbeat_required','plugin_version']
     value={'heartbeat_required':["true","false",True,False],'status':[0,1]}
     for field in mandatory:
        if field not in result:
            obj['Errors']=obj['Errors']+"# Mandatory field "+field+" is missing #"
     for field,val in value.items():
        if field in result and result[field] not in val:
            obj['Errors']=obj['Errors']+"# "+field+" can only be "+str(val)
     if 'plugin_version' in result and not isinstance(result['plugin_version'],int):
        obj['Errors']=obj['Errors']+"# Mandatory field plugin_version should be an integer #"
     RESERVED_KEYS='plugin_version|heartbeat_required|status|units|msg|onchange|display_name|AllAttributeChart'
     attributes_List=[]
     for key,value in result.items():
         if key not in RESERVED_KEYS:
            attributes_List.append(key)
     if len(attributes_List) == 0:
        obj['Errors']="# There should be atleast one \"Number\" type data metric present #"
     if obj['Errors'] !="":
        obj['Result']="**************Plugin output is not valid************"
     else:
        obj['Result']="**************Plugin output is valid**************"
        del obj['Errors']
     result['validation output']= obj